<?php
/**
 * creates an order in WooCommerce - and optionally a customer in WP
 */

## BEGIN PRO ##

class WPL_WooOrderBuilder {

	var $id;
	var $vat_enabled = false;
	var $vat_total   = 0;

	function __construct() {
		global $wpl_logger;
		$this->logger = &$wpl_logger;
	}

	//
	// update woo order from ebay order
	// 
	function updateOrderFromEbayOrder( $id, $post_id = false ) {
		// $this->logger->info( 'updateOrderFromEbayOrder #'.$id );

		// get order details
		$ordersModel = new EbayOrdersModel();		
		$item        = $ordersModel->getItem( $id );
		$details     = $item['details'];
		if ( ! $post_id ) $post_id = $item['post_id'];

		// prevent WooCommerce from sending out notification emails when updating order status
		$this->disableEmailNotifications();

		// get order
		$order = new WC_Order( $post_id );


		// update order creation date
		$timestamp     = strtotime($item['date_created'].' UTC');
		$post_date     = $ordersModel->convertTimestampToLocalTime( $timestamp );
		$post_date_gmt = date_i18n( 'Y-m-d H:i:s', $timestamp, true );
		// $post_date  = date_i18n( 'Y-m-d H:i:s', strtotime($item['date_created'].' UTC'), false );

		// update shop_order post object
		$post_data = array(
			'ID' 	 		 => $post_id,
			'post_date' 	 => $post_date, //The time post was made.
			'post_date_gmt'  => $post_date_gmt, //The time post was made, in GMT.
		);
		$updated_post_id = wp_update_post( $post_data );


		// check if shipping address has changed
		$shipping_details = $details->ShippingAddress;
		$billing_details  = $details->ShippingAddress;
		$new_shipping_address = false;
		if ( get_post_meta( $post_id, '_shipping_address_1', true ) != stripslashes( $shipping_details->Street1 ) ) 	$new_shipping_address = true;
		if ( get_post_meta( $post_id, '_shipping_address_2', true ) != stripslashes( $shipping_details->Street2 ) ) 	$new_shipping_address = true;
		if ( get_post_meta( $post_id, '_shipping_postcode',  true ) != stripslashes( $shipping_details->PostalCode ) ) 	$new_shipping_address = true;
		if ( get_post_meta( $post_id, '_billing_address_1',  true ) != stripslashes( $billing_details->Street1 ) ) 	    $new_shipping_address = true;
		if ( get_post_meta( $post_id, '_billing_postcode',   true ) != stripslashes( $billing_details->PostalCode ) ) 	$new_shipping_address = true;

		// never update shipping address for orders with multi leg shipping enabled
		// if ( $details->IsMultiLegShipping ) $this->processMultiLegShipping( $details, $post_id );
		if ( $details->IsMultiLegShipping ) $new_shipping_address = false;

		// update shipping address if required
		if ( $new_shipping_address ) {

			// optional fields
			if ($billing_details->Phone == 'Invalid Request') $billing_details->Phone = '';
			update_post_meta( $post_id, '_billing_phone', stripslashes( $billing_details->Phone ));

			// billing address
			@list( $billing_firstname, $billing_lastname )     = explode( " ", $billing_details->Name, 2 );
			update_post_meta( $post_id, '_billing_first_name', 	stripslashes( $billing_firstname ) );
			update_post_meta( $post_id, '_billing_last_name', 	stripslashes( $billing_lastname ) );
			update_post_meta( $post_id, '_billing_company', 	stripslashes( $billing_details->CompanyName ) );
			update_post_meta( $post_id, '_billing_address_1', 	stripslashes( $billing_details->Street1 ) );
			update_post_meta( $post_id, '_billing_address_2', 	stripslashes( $billing_details->Street2 ) );
			update_post_meta( $post_id, '_billing_city', 		stripslashes( $billing_details->CityName ) );
			update_post_meta( $post_id, '_billing_postcode', 	stripslashes( $billing_details->PostalCode ) );
			update_post_meta( $post_id, '_billing_country', 	stripslashes( $billing_details->Country ) );
			update_post_meta( $post_id, '_billing_state', 		stripslashes( $billing_details->StateOrProvince ) );
			
			// update shipping address
			@list( $shipping_firstname, $shipping_lastname )   = explode( " ", $shipping_details->Name, 2 );
			update_post_meta( $post_id, '_shipping_first_name', stripslashes( $shipping_firstname ) );
			update_post_meta( $post_id, '_shipping_last_name', 	stripslashes( $shipping_lastname ) );
			update_post_meta( $post_id, '_shipping_company', 	stripslashes( $shipping_details->CompanyName ) );
			update_post_meta( $post_id, '_shipping_address_1', 	stripslashes( $shipping_details->Street1 ) );
			update_post_meta( $post_id, '_shipping_address_2', 	stripslashes( $shipping_details->Street2 ) );
			update_post_meta( $post_id, '_shipping_city', 		stripslashes( $shipping_details->CityName ) );
			update_post_meta( $post_id, '_shipping_postcode', 	stripslashes( $shipping_details->PostalCode ) );
			update_post_meta( $post_id, '_shipping_country', 	stripslashes( $shipping_details->Country ) );
			update_post_meta( $post_id, '_shipping_state', 		stripslashes( $shipping_details->StateOrProvince ) );

			// add order note
			$history_message = "Order #$post_id shipping address was modified.";
			$history_details = array( 'post_id' => $post_id );
			$ordersModel->addHistory( $item['order_id'], 'update_order', $history_message, $history_details );
		}
		


		
		// do nothing if order is already marked as completed, refunded, cancelled or failed
		// if ( $order->status == 'completed' ) return $post_id;
		if ( in_array( $order->status, array( 'completed', 'cancelled', 'refunded', 'failed' ) ) ) return $post_id;

		// order status
		if ( ( $item['eBayPaymentStatus'] == 'PayPalPaymentInProcess' ) || ( $details->PaidTime == '' ) ) { 	
			$new_order_status = get_option( 'wplister_unpaid_order_status', 'on-hold' );
		} elseif ( $item['CompleteStatus'] == 'Completed') { 
			$new_order_status = get_option( 'wplister_new_order_status', 'completed' );
		} else {
			$new_order_status = 'pending';
		}

		// update order status
		if ( $order->status != $new_order_status ) {

			$history_message = "Order #$post_id status was updated from {$order->status} to $new_order_status";
			$history_details = array( 'post_id' => $post_id );
			$ordersModel->addHistory( $item['order_id'], 'update_order', $history_message, $history_details );

			$order->update_status( $new_order_status );

		}

		return $post_id;
	} // updateOrderFromEbayOrder()





	//
	// create woo order from ebay order
	// 
	function createWooOrderFromEbayOrder( $id ) {
		global $wpdb;

		// get order details
		$ordersModel = new EbayOrdersModel();		
		$item        = $ordersModel->getItem( $id );
		$details     = $item['details'];

		$timestamp     = strtotime($item['date_created'].' UTC');
		$post_date     = $ordersModel->convertTimestampToLocalTime( $timestamp );
		$post_date_gmt = date_i18n( 'Y-m-d H:i:s', $timestamp, true );
		// $date_created  = $item['date_created'];
		// $post_date_gmt = date_i18n( 'Y-m-d H:i:s', strtotime($item['date_created'].' UTC'), true );
		// $post_date     = date_i18n( 'Y-m-d H:i:s', strtotime($item['date_created'].' UTC'), false );

		// create order comment
		$order_comment  = $details->BuyerCheckoutMessage . "\n";
		$order_comment .= 'eBay ID: ' . $details->BuyerUserID . "\n";
		$order_comment .= 'Order ID: ' . $item['order_id'];
		if ( $details->ShippingDetails->SellingManagerSalesRecordNumber != '' ) {
			$order_comment .= "\n" . 'Sales record ID: ' . $details->ShippingDetails->SellingManagerSalesRecordNumber;
		}

		// Create shop_order post object
		$post_data = array(
			'post_title'     => 'Order &ndash; '.date('F j, Y @ h:i A', strtotime( $post_date ) ),
			'post_content'   => '',
			'post_excerpt'   => stripslashes( $order_comment ),
			'post_date'      => $post_date, //The time post was made.
			'post_date_gmt'  => $post_date_gmt, //The time post was made, in GMT.
			'post_type'      => 'shop_order',
			'comment_status' => 'closed',
			'ping_status'    => 'closed',
			'post_status'    => function_exists('wc_create_order') ? 'wc-pending' : 'publish' // WC2.2 support
		);

		// Insert order into the database
		$post_id = wp_insert_post( $post_data );

		// Update wp_order_id of order record
		$ordersModel->updateWpOrderID( $id, $post_id );	

		// store OrderID to mark order originated on eBay
		update_post_meta( $post_id, '_ebay_order_id', $item['order_id'] );


		/* the following code is inspired by woocommerce_process_shop_order_meta() in writepanel-order_data.php */

		// add order key
		add_post_meta( $post_id, '_order_key', uniqid('order_'), true );

		// update address
		$billing_details = $details->ShippingAddress;
		$shipping_details = $details->ShippingAddress;

		// optional billing address / RegistrationAddress
		// if ( isset( $details->Buyer->RegistrationAddress ) ) {
		// 	$billing_details = $details->Buyer->RegistrationAddress;
		// }

		// optional fields
		if ($billing_details->Phone == 'Invalid Request') $billing_details->Phone = '';
		update_post_meta( $post_id, '_billing_phone', stripslashes( $billing_details->Phone ));

		// billing address
		@list( $billing_firstname, $billing_lastname )     = explode( " ", $billing_details->Name, 2 );
		update_post_meta( $post_id, '_billing_first_name', 	stripslashes( $billing_firstname ) );
		update_post_meta( $post_id, '_billing_last_name', 	stripslashes( $billing_lastname ) );
		update_post_meta( $post_id, '_billing_company', 	stripslashes( $billing_details->CompanyName ) );
		update_post_meta( $post_id, '_billing_address_1', 	stripslashes( $billing_details->Street1 ) );
		update_post_meta( $post_id, '_billing_address_2', 	stripslashes( $billing_details->Street2 ) );
		update_post_meta( $post_id, '_billing_city', 		stripslashes( $billing_details->CityName ) );
		update_post_meta( $post_id, '_billing_postcode', 	stripslashes( $billing_details->PostalCode ) );
		update_post_meta( $post_id, '_billing_country', 	stripslashes( $billing_details->Country ) );
		update_post_meta( $post_id, '_billing_state', 		stripslashes( $billing_details->StateOrProvince ) );
		
		// shipping address
		@list( $shipping_firstname, $shipping_lastname )   = explode( " ", $shipping_details->Name, 2 );
		update_post_meta( $post_id, '_shipping_first_name', stripslashes( $shipping_firstname ) );
		update_post_meta( $post_id, '_shipping_last_name', 	stripslashes( $shipping_lastname ) );
		update_post_meta( $post_id, '_shipping_company', 	stripslashes( $shipping_details->CompanyName ) );
		update_post_meta( $post_id, '_shipping_address_1', 	stripslashes( $shipping_details->Street1 ) );
		update_post_meta( $post_id, '_shipping_address_2', 	stripslashes( $shipping_details->Street2 ) );
		update_post_meta( $post_id, '_shipping_city', 		stripslashes( $shipping_details->CityName ) );
		update_post_meta( $post_id, '_shipping_postcode', 	stripslashes( $shipping_details->PostalCode ) );
		update_post_meta( $post_id, '_shipping_country', 	stripslashes( $shipping_details->Country ) );
		update_post_meta( $post_id, '_shipping_state', 		stripslashes( $shipping_details->StateOrProvince ) );
		
		// order details
		update_post_meta( $post_id, '_billing_email', 		$item['buyer_email']);
		update_post_meta( $post_id, '_customer_user', 		'0' );
		update_post_meta( $post_id, '_order_currency', 		get_woocommerce_currency() ); // TODO
		update_post_meta( $post_id, '_prices_include_tax', 	get_option( 'woocommerce_prices_include_tax' ) );

		// empty defaults
		update_post_meta( $post_id, '_customer_ip_address', '');
		update_post_meta( $post_id, '_customer_user_agent', '');
		update_post_meta( $post_id, '_cart_discount', 		'');
		update_post_meta( $post_id, '_order_discount', 		'');
		update_post_meta( $post_id, '_order_tax', 			'0.00' ); 			
		update_post_meta( $post_id, '_order_shipping_tax', 	'0.00' ); // TODO


		// check selected shipping service
		$shipping_cost   = 0;
		$shipping_method = 'N/A';

		$ShippingServiceSelected = $details->getShippingServiceSelected();
		if ( $ShippingServiceSelected && method_exists($ShippingServiceSelected, 'getShippingServiceCost'))
			$ShippingServiceCost = $ShippingServiceSelected->getShippingServiceCost();

		if ( isset( $ShippingServiceCost ) && $ShippingServiceCost->value )
			$shipping_cost = $ShippingServiceCost->value;

		if ( $ShippingServiceSelected && method_exists($ShippingServiceSelected, 'getShippingService'))
			$shipping_method = $ShippingServiceSelected->getShippingService();

		// get shipping method title
		$sm = new EbayShippingModel();
		$shipping_title = $sm->getTitleByServiceName( $shipping_method );

		// update shipping
		update_post_meta( $post_id, '_order_shipping', 			$shipping_cost );
		update_post_meta( $post_id, '_shipping_method', 		stripslashes( $shipping_method )); // TODO: mapping
		update_post_meta( $post_id, '_shipping_method_title', 	$shipping_title );
		// update_post_meta( $post_id, '_order_shipping', isset($details->ShippingServiceSelected->ShippingServiceCost->value) ? $details->ShippingServiceSelected->ShippingServiceCost->value : '' );


		// Payment method handling
		$payment_method = $details->CheckoutStatus->PaymentMethod;
		// convert some eBay payment methods to WooCommerce equivalents
		// https://developer.ebay.com/DevZone/flex/docs/Reference/com/ebay/shoppingservice/BuyerPaymentMethodCodeType.html
		if ( $payment_method == 'PayPal' ) 						$payment_method = 'paypal';
		if ( $payment_method == 'COD' ) 						$payment_method = 'cod';
		if ( $payment_method == 'MoneyXferAccepted' ) 			$payment_method = 'bacs'; 
		if ( $payment_method == 'MoneyXferAcceptedInCheckout' ) $payment_method = 'bacs'; 
		update_post_meta( $post_id, '_payment_method', stripslashes( $payment_method ) ); 

		$pm = new EbayPaymentModel();
		$payment_title = $pm->getTitleByServiceName( $details->CheckoutStatus->PaymentMethod );
		update_post_meta( $post_id, '_payment_method_title', $payment_title );
	

		// Order Total
		$order_total = $details->Total->value;
		update_post_meta( $post_id, '_order_total', rtrim(rtrim(number_format( $order_total, 4, '.', ''), '0'), '.') );


		// Tax rows (WC 1.x)
		// $order_taxes = array();
		// [...]		
		// update_post_meta( $post_id, '_order_taxes', $order_taxes );
			

		// Order line items
		$this->processOrderLineItems( $details, $post_id );

		// process tax
		$this->processOrderVAT( $post_id );

		// process orders which use Global Shipping Program
		$this->processMultiLegShipping( $details, $post_id );

		// create customer user - if enabled
		if ( get_option( 'wplister_create_customers' ) ) {
			$user_id = $this->addCustomer( $item['buyer_email'], $details );
			update_post_meta( $post_id, '_customer_user', $user_id );
		}

		// support for WooCommerce Sequential Order Numbers Pro 1.5.6
		if ( isset( $GLOBALS['wc_seq_order_number_pro'] ) && method_exists( $GLOBALS['wc_seq_order_number_pro'], 'set_sequential_order_number' ) )
			$GLOBALS['wc_seq_order_number_pro']->set_sequential_order_number( $post_id );


		// prevent WooCommerce from sending out notification emails when updating order status
		$this->disableEmailNotifications();

		// order metadata had been saved, now get it so we can manipulate status
		$order = new WC_Order( $post_id );
		
		// order status
		if ( ( $item['eBayPaymentStatus'] == 'PayPalPaymentInProcess' ) || ( $details->PaidTime == '' ) ) { 	
			$new_order_status = get_option( 'wplister_unpaid_order_status', 'on-hold' );
		} elseif ( $item['CompleteStatus'] == 'Completed') { 
			$new_order_status = get_option( 'wplister_new_order_status', 'completed' );
		} else {
			$new_order_status = 'pending';
		}

		$order->update_status( $new_order_status );


		// allow other developers to post-process orders created by WP-Lister
		// if you hook into this, please check if get_product() actually returns a valid product object
		// WP-Lister might create order line items which do not exist in WooCommerce!
		// 
		// bad code looks like this:
		// $product = get_product( $item['product_id'] );
		// echo $product->get_sku();
		//
		// good code should look like this:
		// $_product = $order->get_product_from_item( $item );
		// if ( $_product->exists() ) { ... };

		do_action( 'wplister_after_create_order_with_nonexisting_items', $post_id );

		return $post_id;

	} // createWooOrderFromEbayOrder()



	function processOrderVAT( $post_id ) {
		global $wpdb;
		if ( ! $this->vat_enabled ) return;

		// get tax rate
		$tax         = new WC_Tax();
		$tax_rate_id = get_option( 'wplister_process_order_tax_rate_id' );
		$tax_rate    = $wpdb->get_row( "SELECT tax_rate_id, tax_rate_country, tax_rate_state, tax_rate_name, tax_rate_priority FROM {$wpdb->prefix}woocommerce_tax_rates WHERE tax_rate_id = '$tax_rate_id'" );

		$tax_name  = __('VAT','wplister');
		$tax_label = $tax_rate_id ? $tax_rate->tax_rate_name : WC()->countries->tax_or_vat();

		$item_id = woocommerce_add_order_item( $post_id, array(
	 		'order_item_name' 		=> $tax_name,
	 		'order_item_type' 		=> 'tax'
	 	) );

	 	// Add line item meta
	 	if ( $item_id ) {
		 	woocommerce_add_order_item_meta( $item_id, 'compound', 0 );
		 	woocommerce_add_order_item_meta( $item_id, 'tax_amount', $this->format_decimal( $this->vat_total ) );
		 	woocommerce_add_order_item_meta( $item_id, 'shipping_tax_amount', $this->format_decimal( '0.00' ) );

		 	if ( $tax_rate_id ) {
		 		woocommerce_add_order_item_meta( $item_id, 'rate_id', $tax_rate_id );
		 		woocommerce_add_order_item_meta( $item_id, 'label', $tax_label );
		 	}
		}

		// store total order tax
		update_post_meta( $post_id, '_order_tax', $this->format_decimal( $this->vat_total ) ); 			
		// update_post_meta( $post_id, '_order_shipping_tax', $this->format_decimal( '0.00' ) ); 			

	} // processOrderVAT()


	function createOrderLineItem( $Transaction, $post_id ) {

		// get listing item from db
		$listingsModel = new ListingsModel();
		$listingItem = $listingsModel->findItemByEbayID( $Transaction->Item->ItemID );

		$this->logger->info( 'createOrderLineItem #'.$post_id );
		// $this->logger->info( 'createOrderLineItem - listingItem: '.print_r($listingItem,1) );
		// $this->logger->info( 'createOrderLineItem - Transaction: '.print_r($Transaction,1) );

		$product_id			= $listingItem ? $listingItem->post_id : $Transaction->Item->ItemID;
		$item_name 			= $listingItem ? $listingItem->auction_title : $Transaction->Item->Title;

		$item_quantity 		= $Transaction->QuantityPurchased;
		$line_subtotal		= $item_quantity * $Transaction->TransactionPrice->value;
		$line_total 		= $item_quantity * $Transaction->TransactionPrice->value;

		// default to no tax
		$line_subtotal_tax	= '0.00';
		$line_tax		 	= '0.00';
		$item_tax_class		= '';


		// check if listing has VAT enabled
		$vat_enabled		= $listingItem && $listingItem->profile_data['details']['tax_mode'] == 'fix' ? true : false;
		$vat_percent		= $vat_enabled && $listingItem->profile_data['details']['vat_percent'] ? $listingItem->profile_data['details']['vat_percent'] : false;

		// regard global VAT processing setting
		if ( get_option( 'wplister_process_order_vat', 1 ) == 0 ) {
			$vat_enabled = false;
		}

		// process VAT if enabled
		if ( $vat_enabled ) {
			$this->logger->info( 'VAT%: '.$vat_percent );

			// calculate VAT included in line total
			// $vat_tax = $line_total * $vat_percent / 100; 					// calc VAT from net amount
			$vat_tax = $line_total / ( 1 + ( 1 / ( $vat_percent / 100 ) ) );	// calc VAT from gross amount
			// $this->logger->info( 'VAT: '.$vat_tax );

			// keep record of total VAT
			$this->vat_enabled = true;
			$this->vat_total  += $vat_tax;

			// $vat_tax = wc_round_tax_total( $vat_tax );
			$vat_tax = $this->format_decimal( $vat_tax );
			$this->logger->info( 'VAT: '.$vat_tax );

			$line_subtotal_tax	= $vat_tax;
			$line_tax		 	= $vat_tax;

			// adjust item price if prices include tax
			if ( get_option( 'woocommerce_prices_include_tax' ) == 'yes' ) {
				$line_total    = $line_total    - $vat_tax;
				$line_subtotal = $line_subtotal - $vat_tax;
			}

			// try to get product object to set tax class
			$_product = get_product( $listingItem->post_id );

			// set tax class
			if ( $_product && is_object($_product) ) 
				$item_tax_class		= $_product->get_tax_class();

			$this->logger->info( 'tax_class: '.$item_tax_class );
		}


		// check if item has variation 
		$isVariation        = false;
		$VariationSKU       = false;
		$VariationSpecifics = array();
        if ( is_object( @$Transaction->Variation ) ) {
            foreach ($Transaction->Variation->VariationSpecifics as $spec) {
                $VariationSpecifics[ $spec->Name ] = $spec->Value[0];
            }
			$isVariation  = true;
			$VariationSKU = $Transaction->Variation->SKU;
        } 

		// get variation_id
		if ( $isVariation ) {
			$variation_id = ProductWrapper::findVariationID( $product_id, $VariationSpecifics, $VariationSKU );
		}



		$order_item = array();

		$order_item['product_id'] 			= $product_id;
		$order_item['variation_id'] 		= isset( $variation_id ) ? $variation_id : '';
		$order_item['name'] 				= $item_name;
		// $order_item['tax_class']			= $_product->get_tax_class();
		$order_item['tax_class']			= $item_tax_class;
		$order_item['qty'] 					= $item_quantity;
		$order_item['line_subtotal'] 		= $this->format_decimal( $line_subtotal );
		$order_item['line_subtotal_tax'] 	= $line_subtotal_tax;
		$order_item['line_total'] 			= $this->format_decimal( $line_total );
		$order_item['line_tax'] 			= $line_tax;

		// Add line item
	   	$item_id = woocommerce_add_order_item( $post_id, array(
	 		'order_item_name' 		=> $order_item['name'],
	 		'order_item_type' 		=> 'line_item'
	 	) );

	 	// Add line item meta
	 	if ( $item_id ) {
		 	woocommerce_add_order_item_meta( $item_id, '_qty', 				$order_item['qty'] );
		 	woocommerce_add_order_item_meta( $item_id, '_tax_class', 		$order_item['tax_class'] );
		 	woocommerce_add_order_item_meta( $item_id, '_product_id', 		$order_item['product_id'] );
		 	woocommerce_add_order_item_meta( $item_id, '_variation_id', 	$order_item['variation_id'] );
		 	woocommerce_add_order_item_meta( $item_id, '_line_subtotal', 	$order_item['line_subtotal'] );
		 	woocommerce_add_order_item_meta( $item_id, '_line_subtotal_tax',$order_item['line_subtotal_tax'] );
		 	woocommerce_add_order_item_meta( $item_id, '_line_total', 		$order_item['line_total'] );
		 	woocommerce_add_order_item_meta( $item_id, '_line_tax', 		$order_item['line_tax'] );
	 	}

	 	// add variation attributes as order item meta (WC2.2)
	 	if ( $item_id && $isVariation ) {
	 		foreach ($VariationSpecifics as $attribute_name => $value) {
			 	woocommerce_add_order_item_meta( $item_id, $attribute_name,	$value ); 			
	 		}
	 	}

	} // createOrderLineItem()


	function processOrderLineItems( $Details, $post_id ) {

		// WC 2.0 only
		if ( ! function_exists('woocommerce_add_order_item_meta') ) return;

		foreach ( $Details->TransactionArray as $Transaction ) {
			$this->createOrderLineItem( $Transaction, $post_id );
		}
		 
	} // processOrderLineItems()


	function processMultiLegShipping( $details, $post_id ) {

		// check if multi leg shipping is enabled
		if ( ! get_option( 'wplister_process_multileg_orders', 0 ) ) return;
		if ( ! $details->getIsMultiLegShipping() ) return;

		// shortcuts
		$ShipToAddress          = $details->MultiLegShippingDetails->SellerShipmentToLogisticsProvider->ShipToAddress;
		$ShippingServiceDetails = $details->MultiLegShippingDetails->SellerShipmentToLogisticsProvider->ShippingServiceDetails;
		// echo "<pre>";print_r($ShipToAddress);echo"</pre>";
		// echo "<pre>";print_r($ShippingServiceDetails);echo"</pre>";


		// shipping address
		@list( $shipping_firstname, $shipping_lastname )   = explode( " ", $ShipToAddress->Name, 2 );
		update_post_meta( $post_id, '_shipping_first_name', stripslashes( $shipping_firstname ) );
		update_post_meta( $post_id, '_shipping_last_name', 	stripslashes( $shipping_lastname ) );
		update_post_meta( $post_id, '_shipping_company', 	stripslashes( '' ) ); 						// could be "eBay"...
		update_post_meta( $post_id, '_shipping_address_1', 	stripslashes( 'Reference# '.$ShipToAddress->ReferenceID ) );
		update_post_meta( $post_id, '_shipping_address_2', 	stripslashes( $ShipToAddress->Street1 ) );
		update_post_meta( $post_id, '_shipping_city', 		stripslashes( $ShipToAddress->CityName ) );
		update_post_meta( $post_id, '_shipping_postcode', 	stripslashes( $ShipToAddress->PostalCode ) );
		update_post_meta( $post_id, '_shipping_country', 	stripslashes( $ShipToAddress->Country ) );
		update_post_meta( $post_id, '_shipping_state', 		stripslashes( $ShipToAddress->StateOrProvince ) );


		// shipping service
		$shipping_cost   = 0;
		$shipping_method = 'N/A';

		if ( $ShippingServiceDetails && method_exists($ShippingServiceDetails, 'getTotalShippingCost'))
			$TotalShippingCost = $ShippingServiceDetails->getTotalShippingCost();

		if ( isset( $TotalShippingCost ) && $TotalShippingCost->value )
			$shipping_cost = $TotalShippingCost->value;

		if ( $ShippingServiceDetails && method_exists($ShippingServiceDetails, 'getShippingService'))
			$shipping_method = $ShippingServiceDetails->getShippingService();

		// get shipping method title
		$sm = new EbayShippingModel();
		$shipping_title = $sm->getTitleByServiceName( $shipping_method );

		// update shipping
		update_post_meta( $post_id, '_order_shipping', 			$shipping_cost );
		update_post_meta( $post_id, '_shipping_method', 		stripslashes( $shipping_method )); // TODO: mapping
		update_post_meta( $post_id, '_shipping_method_title', 	$shipping_title );


		// fix order total (which should not include the shipping fee eBay charges the buyer)
		$order_total = $details->Subtotal->value + $shipping_cost;
		update_post_meta( $post_id, '_order_total', rtrim(rtrim(number_format( $order_total, 4, '.', ''), '0'), '.') );

	} // processMultiLegShipping()


	function format_decimal( $number ) {

		// wc_format_decimal() exists in WC 2.1+ only
		if ( function_exists('wc_format_decimal') ) 
			return wc_format_decimal( $number );

		$dp     = get_option( 'woocommerce_price_num_decimals' );
		$number = number_format( floatval( $number ), $dp, '.', '' );
		return $number;
		 
	} // format_decimal()



	/**
	 * addCustomer, adds a new customer to newsletter subscriptions
	 *
	 * @param unknown $customers_name
	 * @return $customers_id
	 */
	public function addCustomer( $user_email, $details ) {
		global $wpdb;
		// $this->logger->info( "addCustomer() - data: ".print_r($details,1) );

		// skip if user_email exists
		if ( $user_id = email_exists( $user_email ) ) {
			// $this->show_message('Error: email already exists: '.$user_email, 1 );
			$this->logger->info( "email already exists $user_email" );
			return $user_id;
		}

		// get user data
		$ebay_user_id    = $details->BuyerUserID;
		$user_name       = $details->BuyerUserID;

		// get shipping address with first and last name
		$shipping_details = $details->ShippingAddress;
		@list( $shipping_firstname, $shipping_lastname ) = explode( " ", $shipping_details->Name, 2 );
		$user_firstname  = sanitize_user( $shipping_firstname, true );
		$user_lastname   = sanitize_user( $shipping_lastname, true );
		$user_fullname   = sanitize_user( $shipping_details->Name, true );

		// generate password
		$random_password = wp_generate_password( $length=12, $include_standard_special_chars=false );


		// check if user with ebay account as username exists
		$user_id = username_exists( $ebay_user_id );
		if ( $user_id ) {
			$this->logger->info( "user already exists: $user_name - $user_email ($user_id) " );
			return $user_id;
		}

		// create wp_user
		$wp_user = array(
			'user_login' => $user_name,
			'user_email' => $user_email,
			'first_name' => $user_firstname,
			'last_name' => $user_lastname,
			// 'user_registered' => date( 'Y-m-d H:i:s', strtotime($customer['customers_info_date_account_created']) ),
			'user_pass' => $random_password,
			'role' => 'customer'
			);
		$user_id = wp_insert_user( $wp_user ) ;

		if ( is_wp_error($user_id)) {

			$this->logger->error( 'error creating user '.$user_email.' - WP said: '.$user_id->get_error_message() );
			return false;

		} else {

			// add user meta
			update_user_meta( $user_id, '_ebay_user_id', 		$ebay_user_id );
			update_user_meta( $user_id, 'billing_email', 		$user_email );
			update_user_meta( $user_id, 'paying_customer', 		1 );
			
			// optional phone number
			if ($shipping_details->Phone == 'Invalid Request') $shipping_details->Phone = '';
			update_user_meta( $user_id, 'billing_phone', 		stripslashes( $shipping_details->Phone ));

			// billing
			update_user_meta( $user_id, 'billing_first_name', 	$user_firstname );
			update_user_meta( $user_id, 'billing_last_name', 	$user_lastname );
			update_user_meta( $user_id, 'billing_company', 		stripslashes( $shipping_details->CompanyName ) );
			update_user_meta( $user_id, 'billing_address_1', 	stripslashes( $shipping_details->Street1 ) );
			update_user_meta( $user_id, 'billing_address_2', 	stripslashes( $shipping_details->Street2 ) );
			update_user_meta( $user_id, 'billing_city', 		stripslashes( $shipping_details->CityName ) );
			update_user_meta( $user_id, 'billing_postcode', 	stripslashes( $shipping_details->PostalCode ) );
			update_user_meta( $user_id, 'billing_country', 		stripslashes( $shipping_details->Country ) );
			update_user_meta( $user_id, 'billing_state', 		stripslashes( $shipping_details->StateOrProvince ) );
			
			// shipping
			update_user_meta( $user_id, 'shipping_first_name', 	$user_firstname );
			update_user_meta( $user_id, 'shipping_last_name', 	$user_lastname );
			update_user_meta( $user_id, 'shipping_company', 	stripslashes( $shipping_details->CompanyName ) );
			update_user_meta( $user_id, 'shipping_address_1', 	stripslashes( $shipping_details->Street1 ) );
			update_user_meta( $user_id, 'shipping_address_2', 	stripslashes( $shipping_details->Street2 ) );
			update_user_meta( $user_id, 'shipping_city', 		stripslashes( $shipping_details->CityName ) );
			update_user_meta( $user_id, 'shipping_postcode', 	stripslashes( $shipping_details->PostalCode ) );
			update_user_meta( $user_id, 'shipping_country', 	stripslashes( $shipping_details->Country ) );
			update_user_meta( $user_id, 'shipping_state', 		stripslashes( $shipping_details->StateOrProvince ) );
			
			$this->logger->info( "added customer $user_id ".$user_email." ($ebay_user_id) " );

		}

		return $user_id;

	} // addCustomer()


	function disableEmailNotifications() {

		// prevent WooCommerce from sending out notification emails when updating order status
		if ( get_option( 'wplister_disable_new_order_emails' ) )
			add_filter( 'woocommerce_email_enabled_new_order', array( $this, 'returnFalse' ), 10, 2 );
		if ( get_option( 'wplister_disable_completed_order_emails' ) )
			add_filter( 'woocommerce_email_enabled_customer_completed_order', array( $this, 'returnFalse' ), 10, 2 );
		if ( get_option( 'wplister_disable_processing_order_emails' ) )
			add_filter( 'woocommerce_email_enabled_customer_processing_order', array( $this, 'returnFalse' ), 10, 2 );

	}

	function returnFalse( $param1, $param2 = false ) {
		return false;
	}


    

} // class WPL_WooOrderBuilder
// $WPL_WooOrderBuilder = new WPL_WooOrderBuilder();

## END PRO ##


