<?php
/**
 * custom metabox for woocommerce orders created by WP-Lister Pro
 */

## BEGIN PRO ##

class WpLister_Order_MetaBox {

	var $providers;

	/**
	 * Constructor
	 */
	function __construct() {

		$this->providers = array(
			'Australia Post'         => 'Australia Post',
			'Canada Post'            => 'Canada Post',
			'Chronopost'             => 'Chronopost',
			'City Link'              => 'City Link',
			'ColiposteDomestic'      => 'Coliposte Domestic',
			'ColiposteInternational' => 'Coliposte International',
			'Correos'                => 'Correos',
			'DHL'                    => 'DHL',
			'DPD'                    => 'DPD',
			'DTDC'                   => 'DTDC',
			'FedEx'                  => 'Fedex',
			'GLS'                    => 'GLS',
			'Hermes'                 => 'Hermes',
			'iLoxx'                  => 'iLoxx',
			'Nacex'                  => 'Nacex',
			'OnTrac'                 => 'OnTrac',
			'ParcelForce'            => 'ParcelForce',
			'PostNL'                 => 'PostNL',
			'Posten AB'              => 'Posten AB',
			'Royal Mail'             => 'Royal Mail',
			'SAPO'                   => 'SAPO',
			'TNT'                    => 'TNT',
			'UPS'                    => 'UPS',
			'USPS'                   => 'U.S. Postal Service',
			'Other'                  => 'Other postal service'
		);

		// add_action( 'admin_print_styles', array( &$this, 'admin_styles' ) );
		add_action( 'add_meta_boxes', array( &$this, 'add_meta_box' ) );
		// add_action( 'woocommerce_process_shop_order_meta', array( &$this, 'save_meta_box' ), 0, 2 );
        add_action( 'wp_ajax_wpl_update_ebay_feedback', array( &$this, 'update_ebay_feedback' ) ); 

		add_action( 'woocommerce_order_status_completed', array( &$this, 'handle_woocommerce_order_status_update' ), 0, 1 );

	}

	// function admin_styles() {
	// 	wp_enqueue_style( 'shipment_tracking_styles', plugins_url( basename( dirname( __FILE__ ) ) ) . '/assets/css/admin.css' );
	// }

	/**
	 * Add the meta box for shipment info on the order page
	 *
	 * @access public
	 */
	function add_meta_box() {
		global $post;

		// skip if this is not an order created by WP-Lister
		if ( ! isset( $_GET['post'] ) ) return;
		$ebay_transaction_id = get_post_meta( $_GET['post'], '_ebay_transaction_id', true );
		$ebay_order_id 		 = get_post_meta( $post->ID, '_ebay_order_id', true );
		if ( $ebay_transaction_id ) {

			$title = __('eBay', 'wplister') . ' <small style="color:#999"> &ndash; Transaction ID ' . $ebay_transaction_id . '</small>';
			add_meta_box( 'woocommerce-ebay-details', $title, array( &$this, 'meta_box' ), 'shop_order', 'side', 'core');

		} elseif ( $ebay_order_id ) {

			$title = __('eBay', 'wplister') . ' <small style="color:#999"> #' . $ebay_order_id . '</small>';
			add_meta_box( 'woocommerce-ebay-details', $title, array( &$this, 'meta_box' ), 'shop_order', 'side', 'core');
			
		}

	}

	/**
	 * Show the meta box for shipment info on the order page
	 *
	 * @access public
	 */
	function meta_box() {
		global $woocommerce, $post;

		// $ebay_transaction_id = get_post_meta( $post->ID, '_ebay_transaction_id', true );
		// echo '<p>Transaction ID: '.$ebay_transaction_id.'</p>';
		
		$selected_provider  = get_post_meta( $post->ID, '_tracking_provider', true );
		$shipping_providers = apply_filters( 'wplister_available_shipping_providers', $this->providers );

		echo '<p class="form-field wpl_tracking_provider_field"><label for="wpl_tracking_provider">' . __('Shipping service', 'wplister') . ':</label><br/><select id="wpl_tracking_provider" name="wpl_tracking_provider" class="chosen_select" style="width:100%;">';

		echo '<option value="">-- ' . __('Select shipping service', 'wplister') . ' --</option>';
		foreach ( $shipping_providers as $provider => $display_name  ) {
			echo '<option value="' . $provider . '" ' . selected( $provider, $selected_provider, true ) . '>' . $display_name . '</option>';
		}

		echo '</select> ';

		// allow 3rd party code to fill in the tracking number automatically
		$tracking_number = get_post_meta( $post->ID, '_tracking_number', true );
		$tracking_number = apply_filters( 'wplister_set_tracking_number_for_order', $tracking_number, $post->ID );

		woocommerce_wp_text_input( array(
			'id' 			=> 'wpl_tracking_number',
			'label' 		=> __('Tracking ID:', 'wplister'),
			'placeholder' 	=> '',
			'description' 	=> '',
			'value'			=> $tracking_number
		) );

		woocommerce_wp_text_input( array(
			'id' 			=> 'wpl_date_shipped',
			'label' 		=> __('Shipping date:', 'wplister'),
			'placeholder' 	=> '',
			'description' 	=> '',
			'class'			=> 'date-picker-field',
			'value'			=> ( $date = get_post_meta( $post->ID, '_date_shipped', true ) ) ? date( 'Y-m-d', $date ) : ''
		) );

		woocommerce_wp_text_input( array(
			'id' 			=> 'wpl_feedback_text',
			'label' 		=> __('Your feedback:', 'wplister'),
			'placeholder' 	=> '',
			'description' 	=> 'Feedback is always positive.',
			'value'			=> get_post_meta( $post->ID, '_feedback_text', true )
		) );

		// woocommerce_wp_checkbox( array( 'id' => 'wpl_update_ebay_on_save', 'wrapper_class' => 'update_ebay', 'label' => __('Update on save?', 'wplister') ) );


        // $feedback_text = get_post_meta( $post->ID, '_feedback_text', true );
        // if ( $feedback_text ) {
        //     echo '<p>';
        //     echo '<a id="btn_update_again" href="'.$transaction_id.'" target="_blank" class="button">Update again</a>';
        //     echo '</p>';
        // } else {
            echo '<p>';
            echo '<div id="btn_update_ebay_feedback_spinner" style="float:left;display:none"><img src="'.WP_PLUGIN_URL.'/wp-lister/img/ajax-loader-f9.gif"/></div>';
            echo '<a href="#" id="btn_update_ebay_feedback" class="button button-primary">Update on eBay</a>';
            // echo '<a id="btn_update_again" href="#" style="display:none" target="_blank" class="button">Update again</a>';
            echo '<div id="ebay_result_info" class="updated" style="display:none"><p></p></div>';
            echo '</p>';
            // echo "<br><br>";
        // }



        $woocommerce->add_inline_js("

            var wpl_updateEbayFeedback = function ( post_id ) {


                var tracking_provider = jQuery('#wpl_tracking_provider').val();
                var tracking_number = jQuery('#wpl_tracking_number').val();
                var date_shipped = jQuery('#wpl_date_shipped').val();
                var feedback_text = jQuery('#wpl_feedback_text').val();
                
                // load task list
                var params = {
                    action: 'wpl_update_ebay_feedback',
                    order_id: post_id,
                    wpl_tracking_provider: tracking_provider,
                    wpl_tracking_number: tracking_number,
                    wpl_date_shipped: date_shipped,
                    wpl_feedback_text: feedback_text,
                    nonce: 'TODO'
                };
                var jqxhr = jQuery.getJSON( ajaxurl, params )
                .success( function( response ) { 

                    jQuery('#btn_update_ebay_feedback_spinner').hide();

                    if ( response.success ) {

                        // var transaction_id = response.transaction_id;
                        // var logMsg = 'Transaction #'+transaction_id+' was created.';
                        var logMsg = 'Order details were updated on eBay.';
                        jQuery('#ebay_result_info p').html( logMsg );
                        jQuery('#ebay_result_info').slideDown();
                        jQuery('#btn_update_ebay_feedback').hide('fast');
                        jQuery('#btn_update_again').attr('href',response.invoice_url);
                        jQuery('#btn_update_again').show('fast');

                    } else {

                        var logMsg = '<b>There was a problem updating this order on eBay</b><br><br>'+response.error;
                        jQuery('#ebay_result_info p').html( logMsg );
                        jQuery('#ebay_result_info').addClass( 'error' ).removeClass('updated');
                        jQuery('#ebay_result_info').slideDown();

                        jQuery('#btn_update_ebay_feedback').removeClass('disabled');
                    }


                })
                .error( function(e,xhr,error) { 
                    jQuery('#ebay_result_info p').html( 'The server responded: ' + e.responseText + '<br>' );

                    jQuery('#btn_update_ebay_feedback_spinner').hide();
                    jQuery('#btn_update_ebay_feedback').removeClass('disabled');

                    console.log( 'error', xhr, error ); 
                    console.log( e.responseText ); 
                });

            }

            jQuery('#btn_update_ebay_feedback').click(function(){

                var post_id = jQuery('#post_ID').val();

                jQuery('#btn_update_ebay_feedback_spinner').show();
                jQuery(this).addClass('disabled');
                wpl_updateEbayFeedback( post_id );

                return false;
            });
        ");
	
	}

	/**
	 * Order Downloads Save
	 *
	 * Function for processing and storing all order downloads.
	 */
	function save_meta_box( $post_id, $post ) {
		if ( isset( $_POST['wpl_tracking_number'] ) ) {
		/*
			// get field values
			$wpl_tracking_provider		= esc_attr( $_POST['wpl_tracking_provider'] );
			$wpl_tracking_number 		= esc_attr( $_POST['wpl_tracking_number'] );
			$wpl_date_shipped			= esc_attr( strtotime( $_POST['wpl_date_shipped'] ) );
			$wpl_feedback_text 			= esc_attr( $_POST['wpl_feedback_text'] );

			// Update order data
			update_post_meta( $post_id, '_tracking_provider', $wpl_tracking_provider );
			update_post_meta( $post_id, '_tracking_number', $wpl_tracking_number );
			update_post_meta( $post_id, '_date_shipped', $wpl_date_shipped );
			update_post_meta( $post_id, '_feedback_text', $wpl_feedback_text );

			// build array
			$data = array();
			$data['TrackingNumber']  = $wpl_tracking_number;
			$data['TrackingCarrier'] = $wpl_tracking_provider;
			$data['ShippedTime']     = $wpl_date_shipped;
			$data['FeedbackText']    = $wpl_feedback_text;

			// call CompleteOrder on eBay
			if ( ( $wpl_tracking_number != '' ) && 
				 ( $wpl_date_shipped != '' ) && 
				 ( $wpl_feedback_text != '' ) )  {

				// $message = __('Order details were updated on eBay.','wplister');
				// echo '<div id="message" class="" style="display:block !important;"><p>'.$message.'</p></div>';

				global $oWPL_WPLister;
				$oWPL_WPLister->initEC();
				$oWPL_WPLister->EC->completeOrder( $post_id, $data );
				$oWPL_WPLister->EC->closeEbay();
				// $this->showMessage( __('Order details were updated on eBay.','wplister') );
				
				global $wpl_logger;
				$wpl_logger->info( 'save_meta_box' );
			}

		*/
		}
	}

	// handle order status changed to "completed" - and complete ebay order
    public function handle_woocommerce_order_status_update( $post_id ) {

    	// check if auto complete option is enabled
    	if ( get_option( 'wplister_auto_complete_sales' ) != 1 ) return;

    	// check if default status for new created orders is completed - skip further processing if it is
		if ( get_option( 'wplister_new_order_status', 'completed' ) == 'completed' ) return;

    	// check if this order came in from ebay
    	if ( ! get_post_meta( $post_id, '_ebay_order_id', true ) ) return;

		// build array
		$data = array();
		// $data['ShippedTime']  = date('U');
		$data['ShippedTime']     = '_now_';
		$data['FeedbackText']    = get_option( 'wplister_default_feedback_text', '' );

		global $oWPL_WPLister;
		$oWPL_WPLister->initEC();
		$response = $oWPL_WPLister->EC->completeOrder( $post_id, $data );
		$oWPL_WPLister->EC->closeEbay();

		if ( $response->success ) {
			// update_post_meta( $post_id, '_feedback_text', $wpl_feedback_text );
		}

    }

    /**
     * update feedback and tracking details on ebay (ajax)
     */
    function update_ebay_feedback() {

		// get field values
        $post_id 					= $_REQUEST['order_id'];
		$wpl_tracking_provider		= esc_attr( $_REQUEST['wpl_tracking_provider'] );
		$wpl_tracking_number 		= esc_attr( $_REQUEST['wpl_tracking_number'] );
		$wpl_date_shipped			= esc_attr( strtotime( $_REQUEST['wpl_date_shipped'] ) );
		$wpl_feedback_text 			= esc_attr( $_REQUEST['wpl_feedback_text'] );

		// if tracking number is set, but date is missing, set date today.
		if ( trim($wpl_tracking_number) != '' ) {
			if ( $wpl_date_shipped == '' ) $wpl_date_shipped = date('U');
		}

		// build array
		$data = array();
		$data['TrackingNumber']  = trim( $wpl_tracking_number );
		$data['TrackingCarrier'] = trim( $wpl_tracking_provider );
		$data['ShippedTime']     = trim( $wpl_date_shipped );
		$data['FeedbackText']    = trim( $wpl_feedback_text );

		global $oWPL_WPLister;
		$oWPL_WPLister->initEC();
		$response = $oWPL_WPLister->EC->completeOrder( $post_id, $data );
		$oWPL_WPLister->EC->closeEbay();

		// global $wpl_logger;
		// $wpl_logger->info('RESPONSE: '.print_r( $response, 1 ) );			
		// $this->showMessage( __('Order details were updated on eBay.','wplister') );			

		// Update order data if request was successful
		if ( $response->success ) {
			update_post_meta( $post_id, '_tracking_provider', $wpl_tracking_provider );
			update_post_meta( $post_id, '_tracking_number', $wpl_tracking_number );
			update_post_meta( $post_id, '_date_shipped', $wpl_date_shipped );
			update_post_meta( $post_id, '_feedback_text', $wpl_feedback_text );
		}

        $this->returnJSON( $response );
        exit();

    }

    public function returnJSON( $data ) {
        header('content-type: application/json; charset=utf-8');
        echo json_encode( $data );
    }
    

}
$WpLister_Order_MetaBox = new WpLister_Order_MetaBox();

## END PRO ##


